/*
 * Copyright (C) 2000 Robert Fitzsimons
 */

#ifndef _ujvm_classdata_h
#define _ujvm_classdata_h

#include "basetypes.h"
#include "datatypes.h"

struct Class {
	ClassData* classdata;
	kuint flags;
	kuint8* reference_data;
	kuint8* primitive_data;
};

struct Object {
	Class* class;
	kuint flags;
	kuint8* reference_data;
	kuint8* primitive_data;
};

struct ClassData {
	kuint class_handle;
	ClassData* super_classdata;
	kuint flags;
	kuint methoddata_count;
	MethodData** methoddata;
	kuint fielddata_count;
	FieldData** fielddata;
	kuint interfacedata_count;
	InterfaceData** interfacedata;
	kuint class_reference_data_length;
	kuint class_primitive_data_length;
	kuint object_reference_data_length;
	kuint object_primitive_data_length;
};

struct MethodData {
	ClassData* classdata;
	kuint name_handle;
	kuint descriptor_handle;
	kuint flags;
};

struct FieldData {
	ClassData* classdata;
	kuint name_handle;
	kuint descriptor_handle;
	kuint flags;
	kuint offset;
};

struct InterfaceData {
	ClassData* classdata;
};

#define CONSTANT_Resolved 0x00001000
#define CONSTANT_Unicode  0x00000100
#define CONSTANT_Class    0x00000200

#define FLAG_PUBLIC       0x00010000
#define FLAG_PRIVATE      0x00020000
#define FLAG_PROTECTED    0x00040000
#define FLAG_STATIC       0x00080000
#define FLAG_FINAL        0x00100000
#define FLAG_SUPER        0x00200000
#define FLAG_SYNCHRONIZED 0x00200000
#define FLAG_VOLATILE     0x00400000
#define FLAG_TRANSIENT    0x00800000
#define FLAG_NATIVE       0x01000000
#define FLAG_INTERFACE    0x02000000
#define FLAG_ABSTRACT     0x04000000
#define FLAG_STRICT       0x08000000

#define FLAG_SIZE_MASK        0x000000FF
#define FLAG_RETURN_VOID      0x00000100
#define FLAG_RETURN_REFERENCE 0x00000200
#define FLAG_RETURN_VALUE32   0x00000400
#define FLAG_RETURN_VALUE64   0x00000800
#define FLAG_TYPE_REFERENCE   0x00000200

ClassData* find_classdata(kuint class_handle);
void print_classdata(ClassData* classdata);

#endif /* _ujvm_classdata_h */

